//	DG12128.cpp
#include "DG12128.h"
#include "sim_reg.h"
#include "lcdfont.h"
#include "lcdimage.h"
#include <math.h>

BYTE __rgBitMask[] = {0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01};

//	this is the LCD port config for a 
//	factory-fresh MRM
#define MRM_STOCK_CSBAR9 0x00F00800

//$TODO
//	- optimize graphics routines (e.g. sin, sqrt tables)
//

CDg12128::CDg12128()
{
	CommonConstruct();

	m_pInst	=	(LPBYTE)MRM_STOCK_CSBAR9 + 1;
	m_pData	=	(LPBYTE)MRM_STOCK_CSBAR9;

	DisplayOff();
}

CDg12128::CDg12128(LPBYTE pBase)
{
	CommonConstruct();

	m_pInst	=	pBase + 1;
	m_pData	=	pBase;

	DisplayOff();
}

void CDg12128::CommonConstruct(void)
{
	UINT	ui;

	m_pInst			=	NULL;
	m_pData			=	NULL;
	m_bHPitch		=	8;
	m_bVPitch		=	8;
	m_fCursor		=	FALSE;
	m_fBlink		=	FALSE;
	m_eMode			=	modeCharacter;

	_RecalcChars();

	//	config extra chip select CS3 
	//	CS3 should be same as CS9
	//
	//	note: compiler generates crap (i.e. nothing)
	//	if you do this on one line - yay for gnu
	ui	=	SIM_CSPAR0;
	ui	&=	0xFCFF;
	ui	|=	0x0200;
	
	SIM_CSPAR0	=	ui;
	SIM_CSOR3	=	SIM_CSOR9;
	SIM_CSBAR3	=	SIM_CSBAR9;

}

CDg12128::~CDg12128()
{
	DisplayOff();
}

void CDg12128::SetCharacterVPitch(BYTE bPitch)
{
	if(bPitch > 15)
		bPitch	=	15;

	if(bPitch < 1)
		bPitch	=	1;

	m_bVPitch	=	bPitch;
	_RecalcChars();
}

void CDg12128::SetCharacterHPitch(BYTE bPitch)
{
	if(bPitch < 6)
		bPitch	=	6;
	
	if(bPitch > 8)
		bPitch	=	8;

	m_bHPitch	=	bPitch;
	_RecalcChars();
}

void CDg12128::SetMode(Mode m)
{
	m_eMode	=	m;
	Initialize();
	Clear();
}

void CDg12128::EnableCursor(BOOL fCursor)
{
	//$TODO: reset the mode right now (must save mode byte)
	m_fCursor	=	fCursor;
}

void CDg12128::EnableBlink(BOOL fBlink)
{
	//$TODO: reset the mode right now (must save mode byte)
	m_fBlink	=	fBlink;
}

void CDg12128::_RecalcChars(void)
{
	m_nCharsX	=	ePixelsX / m_bHPitch;
	m_nCharsX	-=	m_nCharsX % 2;
	m_nCharsY	=	ePixelsY / m_bVPitch;
	m_nCharsTotal	=	m_nCharsX * m_nCharsY;
}

void CDg12128::_Wait(void)
{
	while(*m_pInst & 0x80);
}

void CDg12128::_Execute(Command c, BYTE bData)
{
	_Wait();
	SetInstValue(c);
	SetDataValue(bData);
}

void CDg12128::DisplayOff(void)
{
	_Execute(cmdSetMode, mbMaster);
}

void CDg12128::Initialize(void)
{
	BOOL	fRet	=	FALSE;
	BYTE	b;

	b	=	mbDispOn | mbMaster;
	
	if(modeGraphic == GetMode())
	{
		b	|=	mbModeGraph;
	}
	else
	{
		if(m_fCursor)
			b	|=	mbCursorOn;

		if(m_fBlink)
			b	|=	mbBlinkOn;
	}

	_Execute(cmdSetMode, b);
	
	//	128x128 display
	_Execute(cmdSetDuty, ePixelsY - 1);

	//	page 0
	_Execute(cmdSetDispAddrL, 0);
	_Execute(cmdSetDispAddrH, 0);

	switch(GetMode())
	{
	case modeCharacter:
		//	character H pitch
		b	=	((m_bVPitch - 1)  & 0x0f) << 4;
		b	|=	((m_bHPitch - 1) & 0x0f);
		_Execute(cmdSetCharPitch, b);

		//	highest *even* number of chars per line
		_Execute(cmdSetNumChars, BYTE(m_nCharsX - 1));

		//	line on which cursor displayed
		_Execute(cmdSetCursorPos, m_bVPitch - 1);

		//	cursor address
		_Execute(cmdSetCursorAddrL, 0);
		_Execute(cmdSetCursorAddrH, 0);
		break;

	case modeGraphic:
		//	1-byte blocks in a 128x128 display
		_Execute(cmdSetCharPitch, ePixelsPerByte - 1);
		_Execute(cmdSetNumChars, (ePixelsX / ePixelsPerByte) - 1);
		break;
	}
}

void CDg12128::Clear(void)
{
	int			i;
	
	//$BUG 
	//$TODO: take page flipping into account
	//$TODO: after paging implemented: should we 
	//	only clear current page, or clear all? 
	//	use a flag or separate func?

	SetCursorAddress(0);

	//	split into two different loops
	//	because later we may want to erase
	//	only certain pages in graphics mode
	if(modeGraphic == GetMode())
	{
		for(i=0; i < eMemoryBytes; i++)
		{
			_Execute(cmdWriteData, 0x00);
		}
	}
	else
	{
		for(i=0; i < eMemoryBytes; i++)
		{
			_Execute(cmdWriteData, ' ');
		}
	}

	SetCursorAddress(0);
}

//	this isn't Basic; page 0 is the first page ;-)
//
USHORT CDg12128::GetPageAddress(BYTE bPage)
{
	USHORT	nAddress;

	switch(m_eMode)
	{
	case modeCharacter:
		nAddress	=	bPage * (GetCharsX() * (GetCharsY() + 1));
		break;

	case modeGraphic:
		nAddress	=	bPage * (ePixelsY * ePixelsX / ePixelsPerByte);
		break;
	}
	
	return nAddress;
}

//$BUG: change func name.
//	cursor position, in the LCD parlance, moves
//	the vertical line on which the cursor appears
//	in char mode. this is not the same as moving the
//	cursor address, which is what this func is doing
//	via x,y coords. I don't particularly like the 
//	lcd terminology, but we should stick with it
//	to reduce confusion to those poor souls who
//	have actually read the lcd spec
//
void CDg12128::SetCursorPosition(BYTE x, BYTE y)
{
	USHORT	nAddr;

	if(modeGraphic == GetMode())
		return;

	nAddr	=	x + y * m_nCharsX;
	
	SetCursorAddress(nAddr);
}

void CDg12128::SetCursorAddress(USHORT nAddr)
{
	_Execute(cmdSetCursorAddrL, BYTE(nAddr));
	_Execute(cmdSetCursorAddrH, BYTE(nAddr >> BITS_PER_BYTE));
}

void CDg12128::SetDisplayAddress(USHORT nAddr)
{
	_Execute(cmdSetDispAddrL, BYTE(nAddr));
	_Execute(cmdSetDispAddrH, BYTE(nAddr >> BITS_PER_BYTE));
}

void CDg12128::WriteString(char* psz)
{
	if(modeGraphic == GetMode())
		return;

	if(NULL == psz)
		return;

	for(;'\0' != *psz; psz++)
	{
		_Execute(cmdWriteData, *psz);
	}
}

//=============================================================================
//
//	GRAPHICS ROUTINES
//
//
USHORT CDg12128::GetPagePixelYOffset(BYTE bPage)
{
	return bPage * ePixelsY;
}

void CDg12128::SetPixel(USHORT x, USHORT y, Color eColor /*= COLOR_BLACK*/)
{
	USHORT	nAddr;
	BYTE	nBit;
	BYTE	bCurrent;
	USHORT	nBytesPerLine	=	ePixelsX / ePixelsPerByte;

	if(modeCharacter == GetMode())
		return;

	if(x >= ePixelsX)
		return;

	//	note: eMemPixelsY very different from ePixelsY
	//	you can use SetPixel to turn on pixels in any part
	//	of display memory, and use page flipping to display
	//	the right piece of memory
	if(y >= eMemPixelsY)
		return;

	//	each byte is a horizontal group of 8 bits
	//	as hardwired in Initialize()
	//$TODO: paging. for now, assume disp addr == 0
	nAddr	=	(y * nBytesPerLine) + (x / ePixelsPerByte);
	nBit	=	x % ePixelsPerByte;

	//$TODO: the display logic kinda sucks for
	//	reading values -- you need to set the cursor,
	//	then perform a dummy read, then the real read
	//	oh, and then the cursor has been incremented,
	//	so for now we just use bset/bclr instructions
	//
	//	would be nice to be able to OR, XOR, and AND
	//	pixels, but it just seems to expensive
	
	SetCursorAddress(nAddr);

	if(COLOR_BLACK == eColor)
	{
		_Execute(cmdBitSet, nBit);
	}
	else
	{
		_Execute(cmdBitClear, nBit);
	}
}

void CDg12128::Line(USHORT x1, USHORT y1, USHORT x2, USHORT y2, Color eColor /*= COLOR_BLACK*/)
{
	float	dx, dy;
	float	mx, my;
	float	h;
	float	xn, yn;
	USHORT	n;

	if((x2 == x1) && (y2 == y1))
		return;

	dx	=	x2 - x1;
	dy	=	y2 - y1;
	
	h	=	sqrt(dx * dx + dy * dy);
	
	mx	=	dx / h;
	my	=	dy / h;

	xn	=	x1;
	yn	=	y1;
	
	for(n = 0; n <= (int)(h); n++)
	{
		SetPixel(USHORT(xn), USHORT(yn), eColor);
		xn	+=	mx;
		yn	+=	my;
	}
}

void CDg12128::MoveTo(USHORT x, USHORT y)
{
	SetCurrentX(x);
	SetCurrentY(y);
}

void CDg12128::LineTo(USHORT x, USHORT y, Color eColor)
{
	Line(GetCurrentX(), GetCurrentY(), x, y, eColor);
}

void CDg12128::Rectangle(USHORT left, USHORT top, USHORT right, USHORT bottom, Color eColor /*= COLOR_BLACK*/, BOOL fFill /*= FALSE*/)
{
	USHORT	temp;
	USHORT	i;

	if(left > right)
	{
		temp	=	left;
		left	=	right;
		right	=	temp;

	}

	if(top > bottom)
	{
		temp	=	top;
		top		=	bottom;
		bottom	=	temp;

	}

	//$note: I'm BLOWING OFF the Windows GDI convention of 
	//	NOT drawing the lower-right boundary of rects
	//	
	//	in other words, this function inclusively draws 
	//	the right,bottom borders

	if(fFill)
	{
		for(i = top; i < bottom; i++)
		{
			Line(left, i, right, i, eColor);
		}
	}
	else
	{
		Line(left, top, right, top, eColor);
		Line(left, top, left, bottom, eColor);
		Line(right, top, right, bottom, eColor);
		Line(left, bottom, right, bottom, eColor);
	}

}

#define __ROUND(x) (USHORT)((float)x + 0.5)

void CDg12128::Ellipse(USHORT xCenter, USHORT yCenter, USHORT xRadius, USHORT yRadius, 
					   Color eColor /*= COLOR_BLACK*/, BOOL fFill /*= FALSE*/)
{
	if(fFill)
	{
		int		x;
		float	y;
		float	asq;

		Line(xCenter, yCenter - yRadius, xCenter, yCenter + yRadius, eColor);
		asq	=	(float)xRadius * (float)xRadius;
		
		for(x = 1; x <= xRadius; x++)
		{
			y	=	(float)yRadius * sqrt(1 - ((x * x) / asq));
			
			Line(xCenter + x, yCenter - y, xCenter + x, yCenter + y, eColor);
			Line(xCenter - x, yCenter - y, xCenter - x, yCenter + y, eColor);
		}
	}
	else
	{
		float	d;
		float	x,y;
		USHORT	x1,y1,x1l,y1l;
		USHORT	x2,y2,x2l,y2l;
		USHORT	x3,y3,x3l,y3l;
		USHORT	x4,y4,x4l,y4l;

		//	go for pi/2 and draw four quadrants (save on sin/cos)
		x1l	=	xCenter + xRadius;	y1l	=	yCenter;
		x2l	=	xCenter - xRadius;	y2l	=	yCenter;
		x3l	=	xCenter + xRadius;	y3l	=	yCenter;
		x4l	=	xCenter - xRadius;	y4l	=	yCenter;

		for(d = 0.0; x1l > xCenter; d += 0.08)
		{
			x	=	float(xRadius) * cos(d);
			y	=	float(yRadius) * sin(d);
			
			//	the ellipses would be a little asymmetric due
			//	to float truncation if we trunc instead of round
			x1	=	xCenter + __ROUND(x);	y1	=	yCenter + __ROUND(y);
			x2	=	xCenter - __ROUND(x);	y2	=	yCenter - __ROUND(y);
			x3	=	xCenter + __ROUND(x);	y3	=	yCenter - __ROUND(y);
			x4	=	xCenter - __ROUND(x);	y4	=	yCenter + __ROUND(y);

			Line(x1l, y1l, x1, y1, eColor);
			Line(x2l, y2l, x2, y2, eColor);
			Line(x3l, y3l, x3, y3, eColor);
			Line(x4l, y4l, x4, y4, eColor);

			x1l	=	x1;	y1l	=	y1;
			x2l	=	x2;	y2l	=	y2;
			x3l	=	x3;	y3l	=	y3;
			x4l	=	x4;	y4l	=	y4;
		}
	}
}

void CDg12128::SetFont(LPLCDFONT pFont)
{
	m_pFont	=	pFont;
}

void CDg12128::DrawCharacter(USHORT left, USHORT top, char c, Color eColor /* = COLOR_BLACK*/, BkMode eBkMode /*= TRANSPARENT*/)
{
	USHORT		x;
	USHORT		y;
	BYTE		bData;
	LPLCDFONT	pFont	=	GetFont();

	if(NULL == pFont)
		return;

	//$TODO //$BUG
	//	Note that the correct implementation
	//	of text modes requires reading the video
	//	memory and either ANDing or ORing. Since it's
	//	horribly expensive to read this LCDs memory, we're
	//	just going to settle for faking it as below

	for(y = 0; y < pFont->nCharHeight; y++)
	{
		//	get font data for this row of the char
		bData	=	pFont->rgData[( (BYTE)c * pFont->nCharHeight) + y];

		for(x = 0; x < pFont->nCharWidth; x++)
		{
			if(__rgBitMask[x] & bData)
			{
				//	this bit is set; always draw the pixel 
				//	in the specified color
				SetPixel(x+left, y+top, eColor);
			}
			else
			{
				//	if bit is NOT set, then draw this pixel in the 
				//	OPPOSITE color if the mode is OPAQUE
				//	this'll look like the characters are blocked
				//	in by the inverse color
				if(OPAQUE == eBkMode)
				{
					if(COLOR_WHITE == eColor)
						SetPixel(x+left, y+top, COLOR_BLACK);
					else
						SetPixel(x+left, y+top, COLOR_WHITE);
				}
			}
		}
	}
}

void CDg12128::DrawText(USHORT left, USHORT top, LPCSTR pszString, Color eColor /*= COLOR_BLACK*/, BkMode eBkMode /*= TRANSPARENT*/)
{
	USHORT		x		=	left;
	LPLCDFONT	pFont	=	GetFont();

	//	if we don't have a font, don't waste time
	if(NULL == pFont)
		return;

	//$TODO: assert
	if(NULL == pszString)
		return;
	
	x	=	left;
	while('\0' != *pszString)
	{
		DrawCharacter(x, top, *pszString, eColor, eBkMode);
		pszString++;
		x += pFont->nCharWidth;
	}
}

//	draw an image using thresholding
//	if the value of the image pixel is >= threshold, 
//		the pixel will not be drawn if in TRANSPARENT mode, or
//		the pixel will be drawn as white if in OPAQUE mode, or
//
//	if < the threshold, the pixel will be draw as black
//
void CDg12128::DrawImage(USHORT left, USHORT top, LPLCDIMAGE pImage, BkMode eBkMode /*= TRANSPARENT*/, 
						 BYTE bThreshold /*= 128*/, BOOL fFlipV /*= FALSE*/)
{
	USHORT	x;
	USHORT	y;
	USHORT	yCorrected;

	//$TODO: assert
	if(NULL == pImage)
		return;

	for(y = 0; y < pImage->nHeight; y++)
	{
		if(fFlipV)
			yCorrected	=	pImage->nHeight - y - 1;
		else
			yCorrected	=	y;

		for(x = 0; x < pImage->nWidth; x++)
		{
			if(pImage->rgData[(y * pImage->nWidth) + x] < bThreshold)
			{
				SetPixel(left + x, top + yCorrected, COLOR_BLACK);
			}
			else
			{
				if(OPAQUE == eBkMode)
					SetPixel(left + x, top + yCorrected, COLOR_WHITE);
			}
		}
	}
}

