//	DG12128.h
#ifndef __DG12128_H__
#define __DG12128_H__

#include "aedef.h"
#include "lcdfont.h"
#include "lcdimage.h"

enum Color{COLOR_BLACK, COLOR_WHITE};	//	pixel color
enum BkMode{TRANSPARENT, OPAQUE};		//	text drawing mode

//====================================
class CDg12128
{
public:
	//	this ctor will assume that the LCD
	//	has the default MRM lcd port configuration
	CDg12128();

	//	use this ctor to instantiate the object
	//	if the LCD port is configured to non-stock
	//	MRM values (i.e. if CSBAR9 is not 0xF00800)
	CDg12128(LPBYTE pBaseAddress);

	inline void CommonConstruct(void);
	virtual ~CDg12128();

public:
	enum
	{
		//	DG-12128 screen pixels
		//	(which differs from memory pixels; see 
		//	eMemPixelsY below)
		ePixelsX	=	128,
		ePixelsY	=	128,
		
		//	total mem size for DG-12128
		//	with LC3664 RAM
		eMemoryBytes =	4 * BYTES_PER_KILOBYTE,

		//	graphics mode will always use 8 pix per byte
		ePixelsPerByte	=	8,
		
		//	pixels can be set anywhere in memory
		eMemPixelsY	=	ePixelsY * 4,
	};

	//	LCD modes
	enum Mode
	{
		modeCharacter,
		modeGraphic
	};

	//	LC7981 commands
	enum Command
	{
		cmdSetMode			=	0x00,
		cmdSetCharPitch		=	0x01,
		cmdSetNumChars		=	0x02,
		cmdSetDuty			=	0x03,
		cmdSetCursorPos		=	0x04,
		cmdSetDispAddrL		=	0x08,
		cmdSetDispAddrH		=	0x09,
		cmdSetCursorAddrL	=	0x0A,
		cmdSetCursorAddrH	=	0x0B,
		cmdWriteData		=	0x0C,
		cmdReadData			=	0x0D,
		cmdBitClear			=	0x0E,
		cmdBitSet			=	0x0F
	};

	//	bit masks for cmdSetMode functions
	enum ModeBits
	{
		mbDispOn	=	0x20,
		mbMaster	=	0x10,
		mbBlinkOn	=	0x08,
		mbCursorOn	=	0x04,
		mbModeGraph	=	0x02,
		mbCGExt		=	0x01
	};

public:
	//	settings
	inline	void	SetCharacterVPitch(BYTE bVPitch);
	inline	void	SetCharacterHPitch(BYTE bHPitch);
	inline	void	SetMode(Mode m);
	inline	void	EnableCursor(BOOL fOn);
	inline	void	EnableBlink(BOOL fBlink);

	//	general
			void	Initialize(void);
			void	Clear(void);
	inline	void	DisplayOff(void);
			USHORT	GetPageAddress(BYTE bPage);
	
	//	character mode routines
			void	SetCursorPosition(BYTE x, BYTE y);
			void	WriteString(char* pszString);

	//$TODO: make these protected??
	inline	void	SetDisplayAddress(USHORT nAddr);
	inline	void	SetCursorAddress(USHORT nAddr);

	//	graphics mode routines
	inline	USHORT	GetPagePixelYOffset(BYTE bPage);
			void	SetPixel(USHORT x, USHORT y, Color eColor = COLOR_BLACK);
			void	Line(USHORT x1, USHORT y1, USHORT x2, USHORT y2, Color eColor = COLOR_BLACK);
	inline	void	LineTo(USHORT x, USHORT y, Color eColor = COLOR_BLACK);
	inline	void	MoveTo(USHORT x, USHORT y);
			void	Rectangle(USHORT left, USHORT top, USHORT right, USHORT bottom, Color eColor = COLOR_BLACK, BOOL fFill = FALSE);
			void	Ellipse(USHORT xCenter, USHORT yCenter, USHORT xRadius, USHORT yRadius, Color eColor = COLOR_BLACK, BOOL fFill = FALSE);
	inline	void	SetFont(LPLCDFONT pFont);
			void	DrawCharacter(USHORT left, USHORT top, char c, Color eColor = COLOR_BLACK, BkMode eBkMode = TRANSPARENT);
			void	DrawText(USHORT left, USHORT top, LPCSTR pszString, Color eColor = COLOR_BLACK, BkMode eBkMode = TRANSPARENT);
			void	DrawImage(USHORT left, USHORT top, LPLCDIMAGE pImage, BkMode eBkMode = TRANSPARENT, BYTE bThreshold = 128, BOOL fFlipV = FALSE);

	inline	USHORT	GetCharsX(void){return m_nCharsX;};
	inline	USHORT	GetCharsY(void){return m_nCharsY;};

protected:
	//	data helpers
	inline	Mode	GetMode(void)			{return m_eMode;};
	inline	BYTE	GetDataValue(void)		{return *m_pData;};
	inline	BYTE	GetInstValue(void)		{return *m_pInst;};
	inline	void	SetDataValue(BYTE b)	{*m_pData = b;};
	inline	void	SetInstValue(BYTE b)	{*m_pInst = b;};
	inline	USHORT	GetCurrentX(void)		{return m_nCurrentX;};
	inline	USHORT	GetCurrentY(void)		{return m_nCurrentY;};
	inline	void	SetCurrentX(USHORT x)	{m_nCurrentX = x;};
	inline	void	SetCurrentY(USHORT y)	{m_nCurrentY = y;};

	inline	LPLCDFONT	GetFont(void)		{return m_pFont;};

	inline	void	_Wait(void);
	inline	void	_Execute(Command c, BYTE bData);
			void	_RecalcChars(void);

protected:
	Mode		m_eMode;
	BYTE		m_bVPitch;
	BYTE		m_bHPitch;
	BOOL		m_fCursor;
	BOOL		m_fBlink;

	USHORT		m_nCharsX;
	USHORT		m_nCharsY;
	UINT		m_nCharsTotal;
				
	USHORT		m_nCurrentX;
	USHORT		m_nCurrentY;

	LCDFONT*	m_pFont;

private:
	volatile LPBYTE	m_pInst;
	volatile LPBYTE	m_pData;
};



#endif //__DG12128_H__
