#include "aedef.h"
#include "led.h"
#include "DG12128.h"
#include "lcdfont.h"
#include "lcdimage.h"

//$HACK: get a makefile!
#include "led.c"
#include "dg12128.cpp"

//	fonts
#include "fonts\cmd8.h"
#include "fonts\cmd9.h"
#include "fonts\norm8.h"
#include "fonts\bold8.h"
#include "fonts\ital8.h"
#include "fonts\serif8.h"

//	images!
#include "images\i_scene.h"
#include "images\yinyang.h"
#include "images\stripes.h"

#define DELAYMS	50
void Delay(UINT nTime)
{
	while(nTime--);
}

void NewTest(void);

void TestCharBasic(void);
void TestDisplayAddr(void);
void TestCursor(void);
void TestPitches(void);
void TestCharPageFlip(void);
void TestGraphicMode(void);
void TestLines(void);
void TestRects(void);
void TestEllipses(void);
void TestFonts(void);
void TestImages(void);
void TestGraphicPageFlip(void);
void TestGreyscaleEllipse(void);
void TestGreyscaleImage(void);


//$HACK: I hate globals, but this makes the plethora of
//	Test...() functions a lot less cumbersome!
CDg12128	lcd(LPBYTE(0x00f00800));

int main(void)
{
	//	turbo on! don't forget now 19200Bd
	*(LPUSHORT)0xfffa04	=	0x7f08;

	led_init();
	led_off(RED_LED);
	led_off(GREEN_LED);

	//	flash to let user know prog is running
	led_on(RED_LED);
	Delay(100 * DELAYMS);
	led_off(RED_LED);

	//==========================================

	lcd.SetMode(CDg12128::modeCharacter);

	TestCharBasic();			NewTest();
	TestDisplayAddr();			NewTest();
	TestCursor();				NewTest();
	TestPitches();				NewTest();
	TestCharPageFlip();			NewTest();

	lcd.SetMode(CDg12128::modeGraphic);

	TestLines();				NewTest();
	TestRects();				NewTest();
	TestEllipses();				NewTest();
	TestFonts();				NewTest();
	TestImages();				NewTest();
	TestGraphicPageFlip();		NewTest();
	TestGreyscaleEllipse();		NewTest();
	TestGreyscaleImage();		NewTest();

	//==========================================
	
	//	greenlight and return to cpu32bug
	led_on(GREEN_LED);
	asm("trap #0");

	return 0;
}

void NewTest(void)
{
	Delay(4000 * DELAYMS);

	led_on(RED_LED);
	
	lcd.Initialize();
	lcd.Clear();
	Delay(100 * DELAYMS);
	
	led_off(RED_LED);
}

void TestCharBasic(void)
{
	lcd.SetCursorPosition(0, 0);
	lcd.WriteString("0,0 hp=8");

	lcd.SetCursorPosition(12, 0);
	lcd.WriteString("12,0");

	lcd.SetCursorPosition(0, 10);
	lcd.WriteString("0,10");

	lcd.SetCursorPosition(5, 5);
	lcd.WriteString("5,5");
}

void TestDisplayAddr(void)
{
	int			i;

	lcd.SetCursorPosition(0, 15);
	lcd.WriteString("scrolling up, up, and away!!!");

	for(i = 0; i <= 15; i++)
	{
		Delay(400 * DELAYMS);
		lcd.SetDisplayAddress(i * lcd.GetCharsX());
	}
	
}

void TestCursor(void)
{
	lcd.EnableCursor(TRUE);
	lcd.EnableBlink(FALSE);
	lcd.Initialize();
	lcd.WriteString(" cursor on, blink off");
	lcd.SetCursorPosition(0, 0);

	Delay(4000 * DELAYMS);

	lcd.EnableCursor(TRUE);
	lcd.EnableBlink(TRUE);
	lcd.Initialize();
	lcd.WriteString(" cursor on, blink on ");
	lcd.SetCursorPosition(0, 0);

	lcd.EnableCursor(FALSE);
	lcd.EnableBlink(FALSE);
}

void TestPitches(void)
{
	lcd.SetCharacterHPitch(6);
	lcd.Initialize();
	lcd.WriteString("H pitch is 6");

	Delay(4000 * DELAYMS);
	lcd.Clear();

	lcd.SetCharacterHPitch(8);
	lcd.Initialize();
	lcd.WriteString("H pitch now 8");

	Delay(4000 * DELAYMS);
	lcd.Clear();

	lcd.SetCharacterVPitch(7);
	lcd.Initialize();
	lcd.WriteString("V pitch is 7, good for cramming data onto the screen");

	Delay(4000 * DELAYMS);
	lcd.Clear();

	lcd.SetCharacterVPitch(15);
	lcd.Initialize();
	lcd.WriteString("V pitch 15 is a hoot but maybe not so useful");
}

void TestCharPageFlip(void)
{
	int		i;
	USHORT	nPage0;
	USHORT	nPage1;

	nPage0	=	lcd.GetPageAddress(0);
	nPage1	=	lcd.GetPageAddress(1);

	lcd.SetCursorAddress(0);
	lcd.WriteString("page uno");

	lcd.SetCursorAddress(nPage1);
	lcd.WriteString("page deux");

	for(i=0;i<4;i++)
	{
		lcd.SetDisplayAddress(nPage0);
		Delay(1000 * DELAYMS);

		lcd.SetDisplayAddress(nPage1);
		Delay(1000 * DELAYMS);
	}

	lcd.SetDisplayAddress(0);
}

void TestLines(void)
{
	lcd.SetPixel(0, 0);
	lcd.SetPixel(3, 0);
	lcd.SetPixel(5, 0);
 	
	//	light pixels
	lcd.SetPixel(0, 0, COLOR_WHITE);	//note:erases above
	lcd.SetPixel(0, 4, COLOR_WHITE);

	//	lines: angle, vert, horz
	lcd.Line(0, 0, 60, 30);
	lcd.Line(60, 30, 0, 80);
	lcd.Line(64, 20, 64, 30);
	lcd.Line(50, 100, 70, 100);
}

void TestRects(void)
{
	//	open rect
	lcd.Rectangle(20, 20, 50, 50);
	
	//	filled rects
	lcd.Rectangle(100, 100, 120, 120, COLOR_BLACK, TRUE);
	lcd.Rectangle(105, 105, 115, 115, COLOR_WHITE, TRUE);

	//	degenerate rects
	lcd.Rectangle(60, 60, 60, 70);
	lcd.Rectangle(80, 80, 90, 80);

	//	unnormalized
	lcd.Rectangle(20, 80, 10, 90);
	lcd.Rectangle(10, 110, 20, 100);
}

void TestEllipses(void)
{
	//	open ellipses
	lcd.Ellipse(60, 60, 60, 30);
	lcd.Ellipse(100, 40, 20, 40);
	
	//	filled ellipses
	lcd.Ellipse(30, 30, 20, 20, COLOR_BLACK, TRUE);
	lcd.Ellipse(30, 30, 10, 10, COLOR_WHITE, TRUE);

	lcd.Ellipse(64, 64, 40, 10, COLOR_BLACK, TRUE);
	lcd.Ellipse(64, 90, 10, 40, COLOR_BLACK, TRUE);
	lcd.Ellipse(110, 110, 4, 4, COLOR_BLACK, TRUE);
}

void TestFonts(void)
{
	LPLCDFONT	rgFonts[] = {pfCmd08, pfCmd09, pfNorm08, pfBold08, pfItal08, pfSerif08};
	int			nNumFonts	=	sizeof(rgFonts)/sizeof(rgFonts[0]);
	int			i;
	int			y;

	lcd.SetFont(pfCmd08);
	lcd.Rectangle(20, 0, 30, 20, COLOR_BLACK, TRUE);
	y	=	0;
	
	lcd.DrawText(2, y, "cmdline 8");
	y	+=	pfCmd08->nCharHeight + 2;

	lcd.DrawText(2, y, "cmdline 8 opaque", COLOR_BLACK, OPAQUE);
	y	+=	pfCmd08->nCharHeight + 2;

	for(i = 0; i < nNumFonts; i++)
	{
		lcd.SetFont(rgFonts[i]);
		
		lcd.DrawText(2, y, "ab12AB*%^!)\x085\x086", COLOR_BLACK, TRANSPARENT);
		
		y	+=	rgFonts[i]->nCharHeight;
		y	+=	2;
	}
}

void TestImages(void)
{
	int			i;

	for(i = 0; i < 3; i++)
	{
		//	this came in as a BMP, so vflip it
		lcd.DrawImage(i * 20, i * 40, piScene, TRANSPARENT, 1, TRUE);
		lcd.DrawImage(128 - i * 20, 128 - i * 40, piYinyang, TRANSPARENT, 1, TRUE);
	}
}

void TestGraphicPageFlip(void)
{
	UINT	i;
	USHORT	nPage0;
	USHORT	nPage1;

	//	display addresses for pages 0, 1
	nPage0	=	lcd.GetPageAddress(0);
	nPage1	=	lcd.GetPageAddress(1);

	//	page 0: ellipse
	lcd.SetDisplayAddress(nPage0);
	lcd.Ellipse(
		64, lcd.GetPagePixelYOffset(0) + 64, 
		40, 40, COLOR_BLACK, TRUE);

	//	page 1: rectangle
	lcd.SetDisplayAddress(nPage1);
	lcd.Rectangle(
		40, lcd.GetPagePixelYOffset(1) + 40, 
		80, lcd.GetPagePixelYOffset(1) + 80, COLOR_BLACK, TRUE);

	//	flip between page 0 and page 1
	for(i = 0; i < 4; i++)
	{
		lcd.SetDisplayAddress(nPage0);
		Delay(1000 * DELAYMS);

		lcd.SetDisplayAddress(nPage1);
		Delay(1000 * DELAYMS);
	}
	
	lcd.SetDisplayAddress(nPage0);
}

void TestGreyscaleEllipse(void)
{
	UINT	i;
	USHORT	nPage0;
	USHORT	nPage1;

	//	display addresses for pages 0, 1
	nPage0	=	lcd.GetPageAddress(0);
	nPage1	=	lcd.GetPageAddress(1);

	//	make a greyscale by fast flipping!
	//	should see a black center and grey outer ring
	//
	//	note that you can use up to four pages on the DG-12128,
	//	allowing 4 greyscales

	//	big ellipse on page 0
	lcd.Ellipse(64, lcd.GetPagePixelYOffset(0) + 64, 40, 40, COLOR_BLACK, TRUE);

	//	smaller ellipse on page 1
	lcd.Ellipse(64, lcd.GetPagePixelYOffset(1) + 64, 20, 20, COLOR_BLACK, TRUE);

	//	flip between pages quickly
	//	you should see a smaller black ellipse
	//	surrounded by a grey annulus
	//
	for(i = 0; i < 500; i++)
	{
		lcd.SetDisplayAddress(nPage0);
		Delay(1 * DELAYMS);
		
		lcd.SetDisplayAddress(nPage1);
		Delay(1 * DELAYMS);
	}
	lcd.SetDisplayAddress(nPage0);
}

void TestGreyscaleImage(void)
{
	//$BUG
	//	OK, this taught me that there're only
	//	2 graphics pages available;
	//	only use <=3 shades in your images if
	//	you want to display greyscale
	
	UINT		i;
	const BYTE	nPages	=	2;
	BYTE		nPage;
	BYTE		rgThresholds[]	=	{1, 128, 192, 255};

	lcd.SetFont(pfCmd08);

	//	draw an image in each of four
	//	pages. each image has a higher
	//	threshold, so if you devote
	//	the same display time to each page
	//	and keep flipping through, you should
	//	see four grey levels. I hope. ;-)
	for(nPage = 0; nPage < nPages; nPage++)
	{
		//	we have four colors in the bmp: 
		//		0		black
		//		128		dk grey
		//		192		lt grey
		//		255		white
		//	

		lcd.SetDisplayAddress(lcd.GetPageAddress(nPage));
		
		lcd.DrawCharacter(nPage * pfCmd08->nCharWidth, lcd.GetPagePixelYOffset(nPage), '0' + nPage);

		lcd.DrawImage(
			45, lcd.GetPagePixelYOffset(nPage) + 45, 
			piStripes, 
			TRANSPARENT, 
			rgThresholds[nPage], 
			TRUE);

		Delay(2000 * DELAYMS);
	}

	//	flip through all pages equally fast
	for(i = 0; i < (nPages * 200); i++)
	{
		lcd.SetDisplayAddress(lcd.GetPageAddress(i % nPages));
		Delay(1 * DELAYMS);
	}
	
	lcd.SetDisplayAddress(lcd.GetPageAddress(0));
}
